from django.core.serializers.json import DjangoJSONEncoder
from django.utils import translation
from django.utils.functional import Promise


class EncoderMixin:
    def __init__(self, *args, **kwargs):
        self.lang_code = kwargs.pop("lang_code", None)
        super().__init__(*args, **kwargs)


class BaseEncoder(EncoderMixin, DjangoJSONEncoder):
    """
    Everything that DjangoJSONEncoder can handle and translations with selected language
    """

    def default(self, o):
        if isinstance(o, Promise):
            with translation.override(self.lang_code):
                return translation.gettext(o)
        return super().default(o)



from django_actionable_messages.mixins import CardElement


class InputMixin(CardElement):
    input_type = None

    def __init__(self, input_id: str = None, title=None, value: str = None, is_required: bool = None, **kwargs):
        self._data = {
            "@type": self.input_type
        }
        super().__init__(**kwargs)
        if input_id is not None:
            self.set_id(input_id)
        if title is not None:
            self.set_title(title)
        if value is not None:
            self.set_value(value)
        if is_required is not None:
            self.set_is_required(is_required)

    def set_id(self, input_id: str):
        self._data["id"] = input_id

    def set_title(self, title):
        self._data["title"] = title

    def set_value(self, value: str):
        self._data["value"] = value

    def set_is_required(self, is_required=True):
        self._data["isRequired"] = is_required



from django_actionable_messages.exceptions import CardException
from django_actionable_messages.message_card.mixins import InputMixin
from django_actionable_messages.message_card.utils import ChoiceStyle


class TextInput(InputMixin):
    input_type = "TextInput"

    def __init__(self, max_length: int = None, is_multiline: bool = False, **kwargs):
        super().__init__(**kwargs)
        if max_length is not None:
            self.set_max_length(max_length)
        self.set_is_multiline(is_multiline)

    def set_max_length(self, max_length: int):
        self._data["maxLength"] = max_length

    def set_is_multiline(self, is_multiline=True):
        self._data["isMultiline"] = is_multiline


class DateInput(InputMixin):
    input_type = "DateInput"

    def __init__(self, include_time: bool = None, **kwargs):
        super().__init__(**kwargs)
        if include_time is not None:
            self.set_include_time(include_time)

    def set_include_time(self, include_time=True):
        self._data["includeTime"] = include_time


class MultiChoiceInput(InputMixin):
    input_type = "MultichoiceInput"

    def __init__(self, choices: list = None, is_multi_select: bool = None, style: ChoiceStyle = None, **kwargs):
        super().__init__(**kwargs)
        if choices:
            self.add_choices(choices)
        if is_multi_select is not None:
            self.set_is_multi_select(is_multi_select)
        if style is not None:
            self.set_style(style)

    def add_choices(self, choices: list):
        for choice in choices:
            self.add_choice(choice)

    def add_choice(self, choice):
        choices = self._data.setdefault("choices", [])
        self._check_choice(choice, choices)
        self._data["choices"].append(choice.as_data())

    def set_is_multi_select(self, is_multi_select=True):
        self._data["isMultiSelect"] = is_multi_select

    def set_style(self, style: ChoiceStyle):
        self._data["style"] = style

    def _check_choice(self, choice, choices):
        choice_value = choice._get_value()
        values_list = (c["value"] for c in choices)
        if choice_value in values_list:
            raise CardException("Choice with this 'value' [{}] already added".format(choice_value))



from typing import Union

from django_actionable_messages.mixins import CardElement


class Header(CardElement):
    def __init__(self, name: str, value: Union[str, int], **kwargs):
        self._data = {
            "name": name,
            "value": value
        }
        super().__init__(**kwargs)



from typing import Union

from django_actionable_messages.adaptive_card.mixins import ActionMixin
from django_actionable_messages.mixins import CardElement


class OpenUrl(ActionMixin):
    action_type = "Action.OpenUrl"

    def __init__(self, url: str, **kwargs):
        super().__init__(**kwargs)
        self.set_url(url)

    def set_url(self, url: str):
        self._data["url"] = url


class Submit(ActionMixin):
    action_type = "Action.Submit"

    def __init__(self, data: Union[str, dict] = None, **kwargs):
        super().__init__(**kwargs)
        if data is not None:
            self.set_data(data)

    def set_data(self, data: Union[str, dict]):
        self._data["data"] = data


class ShowCard(ActionMixin):
    action_type = "Action.ShowCard"

    def __init__(self, card=None, **kwargs):
        super().__init__(**kwargs)
        if card is not None:
            self.set_card(card)

    def set_card(self, card):
        self._data["card"] = card.payload


class TargetElement(CardElement):
    def __init__(self, element_id: str, is_visible: bool = None, **kwargs):
        self._data = {
            "elementId": element_id
        }
        super().__init__(**kwargs)
        if is_visible is not None:
            self.set_is_visible(is_visible)

    def set_element_id(self, element_id: str):
        self._data["elementId"] = element_id

    def set_is_visible(self, value=True):
        self._data["isVisible"] = value


class ToggleVisibility(ActionMixin):
    action_type = "Action.ToggleVisibility"

    def __init__(self, target_elements: list = None, **kwargs):
        super().__init__(**kwargs)
        if target_elements:
            self.set_target_elements(target_elements)

    def set_target_elements(self, elements: list):
        self._data["targetElements"] = []
        for element in elements:
            if isinstance(element, TargetElement):
                self._data["targetElements"].append(element.as_data())
            else:
                self._data["targetElements"].append(element)



from django_actionable_messages.mixins import Card


class CardMixin(Card):
    content_type = ""

    def __init__(self, title=None, subtitle=None, text=None, images=None, buttons=None, **kwargs):
        self._payload = {
             "contentType": self.content_type
        }
        super().__init__(**kwargs)
        if title is not None:
            self.set_title(title)
        if subtitle is not None:
            self.set_subtitle(subtitle)
        if text is not None:
            self.set_text(text)
        if images:
            self.add_images(images)
        if buttons:
            self.add_buttons(buttons)

    def set_title(self, title):
        self._payload.setdefault("content", {})
        self._payload["content"]["title"] = title

    def set_subtitle(self, subtitle):
        self._payload.setdefault("content", {})
        self._payload["content"]["subtitle"] = subtitle

    def set_text(self, text):
        self._payload.setdefault("content", {})
        self._payload["content"]["text"] = text

    def add_images(self, images):
        self._payload.setdefault("content", {})
        self._payload["content"].setdefault("images", [])
        if isinstance(images, (list, set, tuple)):
            self._payload["content"]["images"].extend(self._get_items_list(images))
        else:
            self._payload["content"]["images"].append(images.as_data())

    def add_buttons(self, buttons):
        self._payload.setdefault("content", {})
        self._payload["content"].setdefault("buttons", [])
        if isinstance(buttons, (list, set, tuple)):
            self._payload["content"]["buttons"].extend(self._get_items_list(buttons))
        else:
            self._payload["content"]["buttons"].append(buttons.as_data())



from django_actionable_messages.mixins import CardElement


class ElementMixin(CardElement):
    def __init__(self, is_visible: bool = None, **kwargs):
        super().__init__(**kwargs)
        if is_visible is not None:
            self.set_is_visible(is_visible)

    def set_is_visible(self, visible=True):
        self._data["isVisible"] = visible


class DisplayFormMixin(ElementMixin):
    base_type = ""

    def __init__(self, title=None, item_id=None, **kwargs):
        self._data = {
            "type": self.base_type
        }
        super().__init__(**kwargs)
        if title is not None:
            self.set_title(title)
        if item_id is not None:
            self.set_item_id(item_id)

    def set_title(self, title):
        self._data["title"] = title

    def set_item_id(self, item_id):
        self._data["itemId"] = item_id



from typing import Union

from django_actionable_messages.message_card.utils import OSType
from django_actionable_messages.mixins import CardElement


class Fact(CardElement):
    def __init__(self, name, value, **kwargs):
        self._data = {}
        super().__init__(**kwargs)
        self._data = {
            "name": name,
            "value": value
        }


class HeroImage(CardElement):
    def __init__(self, url: str, title=None, **kwargs):
        self._data = {
            "image": url
        }
        super().__init__(**kwargs)
        if title is not None:
            self.set_title(title)

    def set_url(self, url: str):
        self._data["image"] = url

    def set_title(self, title):
        self._data["title"] = title


class InputChoice(CardElement):
    def __init__(self, name, value: Union[str, int], **kwargs):
        self._data = {
            "value": value
        }
        super().__init__(**kwargs)
        self._set_display(name)

    def _set_display(self, name):
        self._data["display"] = name

    def _get_value(self):
        return self._data["value"]


class ActionTarget(CardElement):
    def __init__(self, os_type: OSType, url: str, **kwargs):
        self._data = {
            "os": os_type,
            "uri": url
        }
        super().__init__(**kwargs)

    def _get_os(self):
        return self._data["os"]



from django_actionable_messages.mixins import CardElement


class OpenUrl(CardElement):
    def __init__(self, title, url: str, **kwargs):
        self._data = {
            "type": "openUrl",
            "title": title,
            "value": url
        }
        super().__init__(**kwargs)


class Image(CardElement):
    def __init__(self, url: str, alt=None, **kwargs):
        self._data = {
            "url": url
        }
        super().__init__(**kwargs)
        if alt is not None:
            self.set_alt(alt)

    def set_alt(self, alt):
        self._data['alt'] = alt



from django_actionable_messages.mixins import CardElement


class Section(CardElement):
    def __init__(self, start_group: bool = False, title=None, text=None, activity_image: str = None,
                 activity_title=None, activity_subtitle=None, activity_text=None, hero_image=None, facts=None,
                 actions=None, **kwargs):
        self._data = {}
        super().__init__(**kwargs)
        if start_group:
            self.set_start_group(start_group)
        if title is not None:
            self.set_title(title)
        if text is not None:
            self.set_text(text)
        if activity_image is not None:
            self.set_activity_image(activity_image)
        if activity_title is not None:
            self.set_activity_title(activity_title)
        if activity_subtitle is not None:
            self.set_activity_subtitle(activity_subtitle)
        if activity_text is not None:
            self.set_activity_text(activity_text)
        if hero_image is not None:
            self.set_hero_image(hero_image)
        if facts:
            self.add_facts(facts)
        if actions:
            self.add_potential_actions(actions)

    def set_start_group(self, start_group=True):
        self._data["startGroup"] = start_group

    def set_title(self, title):
        self._data["title"] = title

    def set_text(self, text):
        self._data["text"] = text

    def set_activity(self, image: str = None, title=None, subtitle=None, text=None):
        if image is not None:
            self.set_activity_image(image)
        if title is not None:
            self.set_activity_title(title)
        if subtitle is not None:
            self.set_activity_subtitle(subtitle)
        if text is not None:
            self.set_activity_text(text)

    def set_activity_image(self, image: str):
        self._data["activityImage"] = image

    def set_activity_title(self, title):
        self._data["activityTitle"] = title

    def set_activity_subtitle(self, subtitle):
        self._data["activitySubtitle"] = subtitle

    def set_activity_text(self, text):
        self._data["activityText"] = text

    def set_hero_image(self, hero_image):
        self._data["heroImage"] = hero_image.as_data()

    def add_facts(self, facts):
        self._data.setdefault("facts", [])
        if isinstance(facts, (list, set, tuple)):
            self._data["facts"].extend(self._get_items_list(facts))
        else:
            self._data["facts"].append(facts.as_data())

    def add_potential_actions(self, actions):
        self._data.setdefault("potentialAction", [])
        if isinstance(actions, (list, set, tuple)):
            self._data["potentialAction"].extend(self._get_items_list(actions))
        else:
            self._data["potentialAction"].append(actions.as_data())
